import React, { cloneElement, createElement, PropTypes } from 'react';
import classnames from 'classnames';
import { assign, cloneDeep } from 'lodash';

import namespace from '../../namespace';
import map from './map';
import Button from '../button';

const classNamespace = namespace('app__components-editor', true);
const noop = function () {}


export default class Editor extends React.Component {
  static propTypes = {
    form: PropTypes.object,
    onSubmit: PropTypes.func,
    onChange: PropTypes.func,
    onVerificate: PropTypes.func,
    onExcept: PropTypes.func,
    valueGetter: PropTypes.func,
    valueSetter: PropTypes.func
  };

  static defaultProps = {
    onSubmit: noop,
    onClear: noop,
    onChange: noop,
    onVerificate: noop,
    onExcept: noop,
    valueGetter: noop,
    valueSetter: noop
  };

  constructor (props) {
    super(props);

    this.state = {
      form: props.form
    };
  }

  shouldComponentUpdate (nextProps, nextState) {
    if (
      nextProps.form === this.state.form &&
      nextState.form === this.state.from
    ) {
      return false;
    }

    return true;
  }

  componentWillReceiveProps (nextProps) {
    if ('form' in nextProps) {
      this.setState({
        form: nextProps.form
      });
    }
  }

  onChange = (li, value, subject) => {
    const { form } = this.state;
    const { onChange } = this.props;

    console.log(value)

    li.value = value;

    const state = cloneDeep(this.state.form)

    this.setState({
      form: state
    });

    onChange(li, value, state, subject, (form) => {
      this.setState({
        form
      });
    });
  }

  onExceptionClose = (li) => {
    li.exception = null;

    this.setState({
      form: cloneDeep(this.state.form)
    });
  }

  onFormSubmit = () => {
    const { onSubmit, onVerificate, onExcept, valueGetter, valueSetter } = this.props;
    const { form } = this.state;
    const { subjects, button, submitting } = form;
    const exceptions = [];
    let isExcepted = false;

    if (submitting) {
      return this;
    }

    subjects.forEach((subject) => {
      const list = Array.isArray(subject.list) ? subject.list : [subject.list];

      list.forEach((li) => {
        if (li.required) {
          if (li.value === '') {
            isExcepted = true;
            li.exception = 'Can\'t be empty';
          
            return exceptions.push(li);
          }
        }

        const exception = onVerificate(li);

        if (exception) {
          isExcepted = true;
          li.exception = exception;

          exceptions.push(li);
        }
      });
    });

    if (isExcepted) {
      this.setState({
        form: cloneDeep(this.state.form)
      });

      onExcept(exceptions);
    } else {
      onSubmit(assign({
        form: this.state.form,
      }, {
        json: () => {
          const { subjects } = this.state.form;
          const json = {};

          subjects.forEach((subject) => {
            let list = subject.list;
            const isArray = Array.isArray(list);
            const name = subject.name;

            list = isArray ? list : [list];

            list.forEach((li) => {
              const value = valueGetter(li.name, li.value) || li.value;
              json[li.name] = value;
            });
          });

          return json; 
        }
      }));
    }
  }

  formRender () {
    const { form } = this.state;
    const { subjects, button } = form;
    const subjectElements = this.subjectsRender(subjects);
    const buttonElement = this.buttonRender(button);

    return (
      <div className={classNamespace('form')}>
        {subjectElements}
        {buttonElement}
      </div>
    );
  }

  listRender (list, disable, name, group) {
    const { valueSetter } = this.props;

    if (!Array.isArray(list)) {
      list = [list];
    }

    return list.map((li, i) => {
      const component = map[li.type];
      let element;

      if (component) {
        const value = valueSetter(li.name, li.value) || li.value;

        li.value = value;

        element = createElement(component, assign({
          disable,
          onChange: value => this.onChange(li, value, group),
          onExceptionClose: () => this.onExceptionClose(li)
        }, li));
      }

      return (
        <div key={i} className={classNamespace('item')}>
          {element}
        </div>
      );
    });
  }

  subjectsRender (subjects) {
    const elements = subjects.map((subject, i) => {
      const { title, desc, list, disable, name } = subject;
      let descElement;

      if (desc) {
       descElement = (
        <div className={classNamespace('subject-desc')}>
          {desc}
        </div>
       ); 
      }

      const classes = classnames({
        [classNamespace('subject', 'disable')]: !!disable,
        [classNamespace('subject')]: true
      });

      return (
        <div className={classes} key={i}>
          <div className={classNamespace('subject-title')}>
            {title}
          </div>

          <div className={classNamespace('subject-content')}>
            {this.listRender(list, disable, name, subject)}
          </div>

          {descElement}
        </div>
      );
    });

    return (
      <div className={classNamespace('subjects')}>
        {elements}
      </div>
    );
  }

  buttonRender (button) {

    return (
      <Button {...button} onClick={this.onFormSubmit} />
    );
  }

  render () {

    return (
      <div className={classNamespace()}>
        {this.formRender()}
        {this.buttonRender()}
      </div>
    );
  }
}

// form
// - subject
//  - title
//  - formItem
//   - type
//   - json
//   - type json
// - button